/*
 * Copyright (C) 2016 Imagination Technologies
 *
 * SPDX-License-Identifier:	GPL-2.0
 */

#ifndef __DT_BINDINGS_CLOCK_BOSTON_CLOCK_H__
#define __DT_BINDINGS_CLOCK_BOSTON_CLOCK_H__

#define BOSTON_CLK_INPUT 0
#define BOSTON_CLK_SYS 1
#define BOSTON_CLK_CPU 2

#endif /* __DT_BINDINGS_CLOCK_BOSTON_CLOCK_H__ */
