/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _NF_CONNTRACK_IPV6_H
#define _NF_CONNTRACK_IPV6_H

extern const struct nf_conntrack_l4proto nf_conntrack_l4proto_icmpv6;

#endif /* _NF_CONNTRACK_IPV6_H*/
